(error-msg v)(error-msg t v)Validate the cron expression v based on the cron type t, which
can be either :unix or :quartz and defaults to the former.
Returns nil if validation succeeds, and an error string otherwise.
Validate the cron expression `v` based on the cron type `t`, which can be either `:unix` or `:quartz` and defaults to the former. Returns `nil` if validation succeeds, and an error string otherwise.
(next-run v)(next-run t v)Get the next execution time for the cron expression v from
now.
Get the next execution time for the cron expression `v` from now.
(next-run-after v dt)(next-run-after t v dt)Get the next execution time for the cron expression v after
date/time dt.
Get the next execution time for the cron expression `v` after date/time `dt`.
(valid? v)(valid? t v)Validate the cron expression v based on the cron type t, which
can be either :unix or :quartz and defaults to the former.
Returns true or false.
Validate the cron expression `v` based on the cron type `t`, which can be either `:unix` or `:quartz` and defaults to the former. Returns true or false.
(validate v)(validate t v)Validate the cron expression v based on the cron type t, which
can be either :unix or :quartz and defaults to the former.
Throws an IllegalArgumentException if validation fails.
Validate the cron expression `v` based on the cron type `t`, which can be either `:unix` or `:quartz` and defaults to the former. Throws an IllegalArgumentException if validation fails.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |